@@ -16,6 +16,7 @@ use std::sync::Arc;
 #[derive(Debug, Default, Serialize, Deserialize)]
 pub struct GameDb {
     pub description: String,
+    pub date: Option<String>,
     pub games: HashMap<String, Game>,
 }
 
@@ -310,24 +311,24 @@ impl Game {
     }
 
     fn verify(&self, game_root: &Path) -> Vec<VerifyFailure<PathBuf>> {
+        use dashmap::DashMap;
         use rayon::prelude::*;
         use std::fs::read_dir;
-        use std::sync::Mutex;
 
         let mut failures = Vec::new();
 
         // turn files on disk into a map, so extra files can be located
-        let mut files_on_disk: HashMap<String, PathBuf> = HashMap::new();
+        let files_on_disk: DashMap<String, PathBuf> = DashMap::new();
 
         if let Ok(dir) = read_dir(game_root) {
             for entry in dir.filter_map(|e| e.ok()) {
-                if let Ok(name) = entry.file_name().into_string() {
-                    if entry.file_type().map(|t| t.is_file()).unwrap_or(false) {
-                        files_on_disk.insert(name, entry.path());
+                match entry.file_name().into_string() {
+                    Ok(name) => {
+                        if entry.file_type().map(|t| t.is_file()).unwrap_or(false) {
+                            files_on_disk.insert(name, entry.path());
+                        }
                     }
-                } else {
-                    // anything that's not UTF-8 is an extra
-                    failures.push(VerifyFailure::Extra(entry.path()));
+                    Err(_) => failures.push(VerifyFailure::Extra(entry.path())),
                 }
             }
         } else if self.parts.is_empty() {
@@ -337,16 +338,12 @@ impl Game {
         }
 
         // verify all game parts
-        let files_on_disk = Mutex::new(files_on_disk);
         failures.extend(
             self.parts
                 .par_iter()
-                .filter_map(|(name, part)| {
-                    if let Some(pathbuf) = files_on_disk.lock().unwrap().remove(name) {
-                        part.verify(pathbuf)
-                    } else {
-                        Some(VerifyFailure::Missing(game_root.join(name)))
-                    }
+                .filter_map(|(name, part)| match files_on_disk.remove(name) {
+                    Some((_, pathbuf)) => part.verify(pathbuf),
+                    None => Some(VerifyFailure::Missing(game_root.join(name))),
                 })
                 .collect::<Vec<VerifyFailure<PathBuf>>>()
                 .into_iter(),
@@ -355,8 +352,6 @@ impl Game {
         // mark any leftover files on disk as extras
         failures.extend(
             files_on_disk
-                .into_inner()
-                .unwrap()
                 .into_iter()
                 .map(|(_, pathbuf)| VerifyFailure::Extra(pathbuf)),
         );
@@ -400,7 +395,7 @@ impl Game {
             let entry_path = entry.path();
             if let Ok(part) = Part::from_path(&entry_path) {
                 if let Some(target_path) = parts.get(&part) {
-                    file_move(&entry_path, &target_path)?;
+                    file_move(&entry_path, target_path)?;
                 }
             }
         }
@@ -474,16 +469,40 @@ impl<P: AsRef<Path>> fmt::Display for VerifyFailure<P> {
     }
 }
 
+#[derive(Clone, Hash, PartialEq, Eq)]
+enum FileId {
+    Posix { dev: u64, ino: u64 },
+    Unknown(PathBuf),
+}
+
+impl FileId {
+    fn new(path: &Path) -> Result<Self, std::io::Error> {
+        #[cfg(target_os = "linux")]
+        use std::os::linux::fs::MetadataExt;
+        #[cfg(target_os = "macos")]
+        use std::os::macos::fs::MetadataExt;
+
+        if cfg!(target_os = "linux") || cfg!(target_os = "macos") {
+            path.metadata().map(|m| Self::Posix {
+                dev: m.st_dev(),
+                ino: m.st_ino(),
+            })
+        } else {
+            Ok(FileId::Unknown(path.to_path_buf()))
+        }
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub enum Part {
-    ROM { sha1: [u8; 20] },
+    Rom { sha1: [u8; 20] },
     Disk { sha1: [u8; 20] },
 }
 
 impl Part {
     #[inline]
     pub fn new_rom(sha1: &str) -> Self {
-        Part::ROM {
+        Part::Rom {
             sha1: parse_sha1(sha1),
         }
     }
@@ -505,7 +524,7 @@ impl Part {
     #[inline]
     pub fn digest(&self) -> Digest {
         match self {
-            Part::ROM { sha1 } => Digest(sha1),
+            Part::Rom { sha1 } => Digest(sha1),
             Part::Disk { sha1 } => Digest(sha1),
         }
     }
@@ -520,13 +539,38 @@ impl Part {
             .and_then(|mut r| Part::from_reader(&mut r))
     }
 
+    fn from_cached_path(path: &Path) -> Result<Self, std::io::Error> {
+        use dashmap::DashMap;
+        use fxhash::FxBuildHasher;
+        use once_cell::sync::OnceCell;
+
+        static PART_CACHE: OnceCell<DashMap<FileId, Part, FxBuildHasher>> = OnceCell::new();
+
+        let file_id = FileId::new(path)?;
+
+        // using DashMap's Entry API leaves the map locked
+        // while generating the Part from path
+        // which locks out other threads until finished
+        // whereas a get()/insert() pair does not
+        let map = PART_CACHE.get_or_init(|| DashMap::default());
+
+        match map.get(&file_id) {
+            Some(part) => Ok(part.clone()),
+            None => {
+                let part = Self::from_path(path)?;
+                map.insert(file_id, part.clone());
+                Ok(part)
+            }
+        }
+    }
+
     fn from_reader<R: Read>(r: R) -> Result<Self, std::io::Error> {
         use std::io::{copy, sink};
 
         let mut r = Sha1Reader::new(r);
         match Part::disk_from_reader(&mut r) {
             Ok(Some(part)) => Ok(part),
-            Ok(None) => copy(&mut r, &mut sink()).map(|_| Part::ROM { sha1: r.digest() }),
+            Ok(None) => copy(&mut r, &mut sink()).map(|_| Part::Rom { sha1: r.digest() }),
             Err(err) => Err(err),
         }
     }
@@ -565,7 +609,7 @@ impl Part {
     }
 
     fn verify<P: AsRef<Path>>(&self, part_path: P) -> Option<VerifyFailure<P>> {
-        match Part::from_path(part_path.as_ref()) {
+        match Part::from_cached_path(part_path.as_ref()) {
             Ok(ref disk_part) if self == disk_part => None,
             Ok(_) => Some(VerifyFailure::Bad(part_path)),
             Err(err) => Some(VerifyFailure::Error(part_path, err)),
@@ -718,25 +762,69 @@ pub enum RomSource {
 impl RomSource {
     pub fn from_path(pb: PathBuf) -> Result<Vec<(Part, RomSource)>, Error> {
         use std::fs::File;
-        use std::io::{BufReader, Cursor};
+        use std::io::{BufReader, Cursor, Seek, SeekFrom};
 
         let mut r = File::open(&pb).map(BufReader::new)?;
 
-        // valid parts might be less than 4 bytes,
-        // so fallback to reading a raw part
-        // if some error occurs testing for zipfiles
+        let mut result = vec![(Part::from_reader(&mut r)?, RomSource::Disk(pb.clone()))];
+
+        // valid ROMs might also be potentially invalid Zip archives
+        // so the extra error handling ensures original file(s)
+        // are returned as part of the result if something goes bad
+        // after the initial read
+
+        if r.seek(SeekFrom::Start(0)).is_err() {
+            return Ok(result);
+        }
+
         if is_zip(&mut r).unwrap_or(false) {
             let file = Arc::new(pb);
-            let mut zip = zip::ZipArchive::new(r)?;
-            let mut result = Vec::new();
+            let mut zip = match zip::ZipArchive::new(r) {
+                Ok(z) => z,
+                Err(_) => return Ok(result),
+            };
+
             for index in 0..zip.len() {
                 let mut file_data = Vec::new();
-                zip.by_index(index)?.read_to_end(&mut file_data)?;
+                match zip.by_index(index) {
+                    Ok(mut r) => {
+                        if r.read_to_end(&mut file_data).is_err() {
+                            return Ok(result);
+                        }
+                    }
+                    Err(_) => return Ok(result),
+                }
+
                 let mut reader = Cursor::new(file_data);
+                if let Ok(part) = Part::from_reader(&mut reader) {
+                    result.push((
+                        part,
+                        RomSource::Zip {
+                            file: file.clone(),
+                            index,
+                        },
+                    ));
+                }
+
+                if reader.seek(SeekFrom::Start(0)).is_err() {
+                    return Ok(result);
+                }
+
                 if is_zip(&mut reader).unwrap_or(false) {
-                    let mut sub_zip = zip::ZipArchive::new(reader)?;
+                    let mut sub_zip = match zip::ZipArchive::new(reader) {
+                        Ok(z) => z,
+                        Err(_) => return Ok(result),
+                    };
+
                     for sub_index in 0..sub_zip.len() {
-                        let part = Part::from_reader(sub_zip.by_index(sub_index)?)?;
+                        let part = match sub_zip.by_index(sub_index) {
+                            Ok(z) => match Part::from_reader(z) {
+                                Ok(part) => part,
+                                Err(_) => return Ok(result),
+                            },
+                            Err(_) => return Ok(result),
+                        };
+
                         result.push((
                             part,
                             RomSource::SubZip {
@@ -746,21 +834,11 @@ impl RomSource {
                             },
                         ))
                     }
-                } else {
-                    let part = Part::from_reader(reader)?;
-                    result.push((
-                        part,
-                        RomSource::Zip {
-                            file: file.clone(),
-                            index,
-                        },
-                    ))
                 }
             }
-            Ok(result)
-        } else {
-            Ok(vec![(Part::from_reader(r)?, RomSource::Disk(pb))])
         }
+
+        Ok(result)
     }
 
     fn extract(&self, target: &Path) -> Result<Extracted, Error> {
