@@ -30,9 +30,9 @@ static CACHE_REDUMP_SPLIT: &str = "redump-split.cbor";
 #[derive(Debug)]
 pub enum Error {
     IO(std::io::Error),
-    XML(roxmltree::Error),
-    SQL(rusqlite::Error),
-    CBOR(serde_cbor::Error),
+    Xml(roxmltree::Error),
+    Sql(rusqlite::Error),
+    Cbor(serde_cbor::Error),
     Zip(zip::result::ZipError),
     NoSuchSoftwareList(String),
     NoSuchSoftware(String),
@@ -47,19 +47,19 @@ impl From<std::io::Error> for Error {
 
 impl From<roxmltree::Error> for Error {
     fn from(err: roxmltree::Error) -> Self {
-        Error::XML(err)
+        Error::Xml(err)
     }
 }
 
 impl From<rusqlite::Error> for Error {
     fn from(err: rusqlite::Error) -> Self {
-        Error::SQL(err)
+        Error::Sql(err)
     }
 }
 
 impl From<serde_cbor::Error> for Error {
     fn from(err: serde_cbor::Error) -> Self {
-        Error::CBOR(err)
+        Error::Cbor(err)
     }
 }
 
@@ -73,9 +73,9 @@ impl std::error::Error for Error {
     fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match self {
             Error::IO(err) => Some(err),
-            Error::XML(err) => Some(err),
-            Error::SQL(err) => Some(err),
-            Error::CBOR(err) => Some(err),
+            Error::Xml(err) => Some(err),
+            Error::Sql(err) => Some(err),
+            Error::Cbor(err) => Some(err),
             Error::Zip(err) => Some(err),
             Error::NoSuchSoftwareList(_) | Error::NoSuchSoftware(_) | Error::MissingCache(_) => {
                 None
@@ -88,9 +88,9 @@ impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             Error::IO(err) => err.fmt(f),
-            Error::XML(err) => err.fmt(f),
-            Error::SQL(err) => err.fmt(f),
-            Error::CBOR(err) => err.fmt(f),
+            Error::Xml(err) => err.fmt(f),
+            Error::Sql(err) => err.fmt(f),
+            Error::Cbor(err) => err.fmt(f),
             Error::Zip(err) => err.fmt(f),
             Error::NoSuchSoftwareList(s) => write!(f, "no such software list \"{}\"", s),
             Error::NoSuchSoftware(s) => write!(f, "no such software \"{}\"", s),
@@ -437,7 +437,7 @@ impl OptMessCreate {
             mess::clear_tables(&trans)?;
             self.xml
                 .iter()
-                .try_for_each(|p| mess::add_file(&p, &trans))?;
+                .try_for_each(|p| mess::add_file(p, &trans))?;
             trans.commit()?;
 
             println!("* Wrote \"{}\"", db_file.display());
@@ -880,9 +880,9 @@ impl OptMessSplit {
                 if let Some(exact_match) = db
                     .possible_matches(data.len() as u64)
                     .iter()
-                    .find(|m| m.matches(&data))
+                    .find(|m| m.matches(data))
                 {
-                    exact_match.extract(&self.output, &data)?;
+                    exact_match.extract(&self.output, data)?;
                 }
             }
 
@@ -973,25 +973,19 @@ impl OptExtraCreate {
         let new_games = self
             .dats
             .into_iter()
-            .filter_map(|dat| read_dat_or_zip(dat).transpose())
-            .collect::<Result<Vec<game::Game>, Error>>()?;
+            .map(read_dat_or_zip)
+            .collect::<Result<Vec<Vec<(String, game::GameDb)>>, Error>>()?;
 
-        let db = if self.append {
-            let mut db = read_cache::<game::GameDb>(EXTRA, CACHE_EXTRA)?;
-            for game in new_games {
-                db.games.insert(game.name.clone(), game);
-            }
-            db
+        let mut db = if self.append {
+            read_cache::<extra::ExtraDb>(EXTRA, CACHE_EXTRA)?
         } else {
-            game::GameDb {
-                description: "MAME Extras".to_string(),
-                games: new_games
-                    .into_iter()
-                    .map(|game| (game.name.clone(), game))
-                    .collect(),
-            }
+            extra::ExtraDb::default()
         };
 
+        for (name, game_db) in new_games.into_iter().flatten() {
+            db.insert(name, game_db);
+        }
+
         write_cache(CACHE_EXTRA, db)
     }
 }
@@ -1001,8 +995,8 @@ struct OptExtraList {}
 
 impl OptExtraList {
     fn execute(self) -> Result<(), Error> {
-        let db = read_cache::<game::GameDb>(EXTRA, CACHE_EXTRA)?;
-        db.list(None, game::GameColumn::Description, false);
+        let db = read_cache::<extra::ExtraDb>(EXTRA, CACHE_EXTRA)?;
+        mess::list_all(&db);
         Ok(())
     }
 }
@@ -1011,11 +1005,18 @@ impl OptExtraList {
 struct OptExtraParts {
     /// sections's parts to search for
     section: String,
+
+    /// section's name to search for
+    name: String,
 }
 
 impl OptExtraParts {
     fn execute(self) -> Result<(), Error> {
-        let db = read_cache::<game::GameDb>(EXTRA, CACHE_EXTRA)?;
+        let db = read_cache::<extra::ExtraDb>(EXTRA, CACHE_EXTRA).and_then(|mut db| {
+            db.remove(&self.name)
+                .ok_or_else(|| Error::NoSuchSoftwareList(self.name.clone()))
+        })?;
+
         db.display_parts(&self.section)
     }
 }
@@ -1026,21 +1027,28 @@ struct OptExtraVerify {
     #[structopt(short = "d", long = "dir", parse(from_os_str), default_value = ".")]
     dir: PathBuf,
 
-    /// extra to verify
-    extras: Vec<String>,
+    /// extras category to verify
+    extra: String,
+
+    /// machine to verify
+    software: Vec<String>,
+
+    /// display only failures
+    #[structopt(long = "failures")]
+    failures: bool,
 }
 
 impl OptExtraVerify {
     fn execute(self) -> Result<(), Error> {
-        let db: game::GameDb = read_cache(EXTRA, CACHE_EXTRA)?;
+        let db = read_cache::<extra::ExtraDb>(EXTRA, CACHE_EXTRA)?
+            .remove(&self.extra)
+            .ok_or_else(|| Error::NoSuchSoftwareList(self.extra.clone()))?;
 
-        let extras: HashSet<String> = if !self.extras.is_empty() {
-            // only validate user-specified extras
-            let extras = self.extras.iter().cloned().collect();
-            db.validate_games(&extras)?;
-            extras
+        let software: HashSet<String> = if !self.software.is_empty() {
+            let software = self.software.iter().cloned().collect();
+            db.validate_games(&software)?;
+            software
         } else {
-            // ignore stuff that's on disk but not valid machines
             self.dir
                 .read_dir()?
                 .filter_map(|e| {
@@ -1051,7 +1059,52 @@ impl OptExtraVerify {
                 .collect()
         };
 
-        verify(&db, &self.dir, &extras, false);
+        verify(&db, &self.dir, &software, self.failures);
+
+        Ok(())
+    }
+}
+
+#[derive(StructOpt)]
+struct OptExtraVerifyAll {
+    /// extras directory
+    #[structopt(short = "d", long = "dir", parse(from_os_str), default_value = ".")]
+    dir: PathBuf,
+
+    /// verify all possible machines
+    #[structopt(long = "all")]
+    all: bool,
+
+    /// display only failures
+    #[structopt(long = "failures")]
+    failures: bool,
+}
+
+impl OptExtraVerifyAll {
+    fn execute(self) -> Result<(), Error> {
+        let db = read_cache::<extra::ExtraDb>(EXTRA, CACHE_EXTRA)?;
+
+        for (extra_list, db) in db.into_iter() {
+            let extras_path = self.dir.join(extra_list);
+
+            let software: HashSet<String> = if self.all {
+                db.all_games()
+            } else {
+                extras_path
+                    .read_dir()
+                    .map(|dir| {
+                        dir.filter_map(|e| {
+                            e.ok()
+                                .and_then(|e| e.file_name().into_string().ok())
+                                .filter(|s| db.is_game(s))
+                        })
+                        .collect()
+                    })
+                    .unwrap_or_default()
+            };
+
+            verify(&db, &extras_path, &software, self.failures);
+        }
 
         Ok(())
     }
@@ -1072,18 +1125,23 @@ struct OptExtraAdd {
     dry_run: bool,
 
     /// extra to add
-    extras: Vec<String>,
+    extra: String,
+
+    /// machine to add
+    software: Vec<String>,
 }
 
 impl OptExtraAdd {
     fn execute(mut self) -> Result<(), Error> {
-        let db: game::GameDb = read_cache(EXTRA, CACHE_EXTRA)?;
+        let db = read_cache::<extra::ExtraDb>(EXTRA, CACHE_EXTRA)?
+            .remove(&self.extra)
+            .ok_or_else(|| Error::NoSuchSoftwareList(self.extra.clone()))?;
 
-        let mut parts = if self.extras.is_empty() {
-            self.extras = db.all_games();
+        let mut parts = if self.software.is_empty() {
+            self.software = db.all_games();
             game::all_rom_sources(&self.input)
         } else {
-            game::get_rom_sources(&self.input, db.required_parts(&self.extras)?)
+            game::get_rom_sources(&self.input, db.required_parts(&self.software)?)
         };
 
         let copy = if self.dry_run {
@@ -1092,12 +1150,47 @@ impl OptExtraAdd {
             game::extract
         };
 
-        self.extras
+        self.software
             .iter()
             .try_for_each(|game| db.games[game].add(&mut parts, &self.dir, copy))
     }
 }
 
+#[derive(StructOpt)]
+struct OptExtraAddAll {
+    /// input directory
+    #[structopt(short = "i", long = "input", parse(from_os_str), default_value = ".")]
+    input: PathBuf,
+
+    /// output directory
+    #[structopt(short = "d", long = "dir", parse(from_os_str), default_value = ".")]
+    dir: PathBuf,
+
+    /// don't actually add files
+    #[structopt(long = "dry-run")]
+    dry_run: bool,
+}
+
+impl OptExtraAddAll {
+    fn execute(self) -> Result<(), Error> {
+        let db = read_cache::<extra::ExtraDb>(EXTRA, CACHE_EXTRA)?;
+
+        let mut parts = game::all_rom_sources(&self.input);
+
+        let copy = if self.dry_run {
+            game::extract_dry_run
+        } else {
+            game::extract
+        };
+
+        db.into_iter().try_for_each(|(extra, db)| {
+            let extras_path = self.dir.join(extra);
+            db.games_iter()
+                .try_for_each(|game| game.add(&mut parts, &extras_path, copy))
+        })
+    }
+}
+
 #[derive(StructOpt)]
 #[structopt(name = "extra")]
 enum OptExtra {
@@ -1120,6 +1213,14 @@ enum OptExtra {
     /// add files to directory
     #[structopt(name = "add")]
     Add(OptExtraAdd),
+
+    /// add all files to directory
+    #[structopt(name = "add-all")]
+    AddAll(OptExtraAddAll),
+
+    /// verify all files in directory
+    #[structopt(name = "verify-all")]
+    VerifyAll(OptExtraVerifyAll),
 }
 
 impl OptExtra {
@@ -1130,6 +1231,8 @@ impl OptExtra {
             OptExtra::Parts(o) => o.execute(),
             OptExtra::Verify(o) => o.execute(),
             OptExtra::Add(o) => o.execute(),
+            OptExtra::AddAll(o) => o.execute(),
+            OptExtra::VerifyAll(o) => o.execute(),
         }
     }
 }
@@ -1155,7 +1258,7 @@ impl OptRedumpCreate {
             redump::clear_tables(&trans)?;
             self.xml
                 .iter()
-                .try_for_each(|p| redump::add_file(&p, &trans))?;
+                .try_for_each(|p| redump::add_file(p, &trans))?;
             trans.commit()?;
 
             println!("* Wrote \"{}\"", db_file.display());
@@ -1194,7 +1297,7 @@ impl OptRedumpList {
         if let Some(software_list) = self.software_list {
             let db = db
                 .remove(&software_list)
-                .ok_or_else(|| Error::NoSuchSoftwareList(software_list))?;
+                .ok_or(Error::NoSuchSoftwareList(software_list))?;
             redump::list(&db, self.search.as_deref())
         } else {
             redump::list_all(&db)
@@ -1399,15 +1502,78 @@ impl OptRedump {
 struct OptIdentify {
     /// ROMs or CHDs to identify
     parts: Vec<PathBuf>,
+
+    /// perform reverse lookup
+    #[structopt(short = "l", long = "lookup")]
+    lookup: bool,
 }
 
 impl OptIdentify {
     fn execute(self) -> Result<(), Error> {
-        use crate::game::RomSource;
+        use crate::game::{GameDb, Part, RomSource};
+        use prettytable::Table;
+        use std::collections::{BTreeMap, HashMap};
 
-        for path in self.parts.into_iter() {
-            for (part, source) in RomSource::from_path(path)? {
-                println!("{}  {}", part.digest(), source);
+        #[inline]
+        fn make_mame(db: game::GameDb) -> BTreeMap<String, GameDb> {
+            let mut map = BTreeMap::default();
+            map.insert("".to_string(), db);
+            map
+        }
+
+        if self.lookup {
+            let all_parts: [(&str, BTreeMap<String, GameDb>); 3] = [
+                (
+                    "mame",
+                    make_mame(read_cache(MAME, CACHE_MAME).unwrap_or_default()),
+                ),
+                ("mess", read_cache(MESS, CACHE_MESS).unwrap_or_default()),
+                ("extra", read_cache(EXTRA, CACHE_EXTRA).unwrap_or_default()),
+            ];
+
+            let mut lookup: HashMap<&Part, Vec<[&str; 4]>> = HashMap::default();
+            for (category, game_dbs) in &all_parts {
+                for (system, game_db) in game_dbs.iter() {
+                    for game in game_db.games_iter() {
+                        for (rom, part) in game.parts.iter() {
+                            lookup.entry(part).or_default().push([
+                                category,
+                                system,
+                                game.name.as_str(),
+                                rom,
+                            ]);
+                        }
+                    }
+                }
+            }
+
+            for path in self.parts.into_iter() {
+                println!("{}:", path.display());
+                for (part, _) in RomSource::from_path(path)? {
+                    match lookup.get(&part) {
+                        Some(sources) => {
+                            use prettytable::{cell, format, row};
+                            let mut table = Table::new();
+                            table.set_format(*format::consts::FORMAT_NO_BORDER_LINE_SEPARATOR);
+                            table.get_format().column_separator('\u{2502}');
+
+                            for [category, system, game, rom] in sources {
+                                table.add_row(row![category, system, game, rom]);
+                            }
+
+                            table.printstd();
+                        }
+                        None => {
+                            println!("unknown");
+                        }
+                    }
+                }
+            }
+        } else {
+            for path in self.parts.into_iter() {
+                for (part, source) in RomSource::from_path(path)? {
+                    println!("{}  {}", part.digest(), source);
+                }
             }
         }
 
@@ -1484,11 +1650,10 @@ fn read_raw_or_zip<P: AsRef<Path>>(file: P) -> Result<String, Error> {
     Ok(data)
 }
 
-// attempts to read the appopriate .dat in a zip file,
+// attempts to read all .dat files in a .zip file
 // or the whole raw file if it is not a .zip
-fn read_dat_or_zip<P: AsRef<Path>>(file: P) -> Result<Option<game::Game>, Error> {
+fn read_dat_or_zip<P: AsRef<Path>>(file: P) -> Result<Vec<(String, game::GameDb)>, Error> {
     let mut f = File::open(file)?;
-    let mut data = String::new();
     if is_zip(&mut f)? {
         use zip::ZipArchive;
 
@@ -1500,29 +1665,19 @@ fn read_dat_or_zip<P: AsRef<Path>>(file: P) -> Result<Option<game::Game>, Error>
             .map(|s| s.to_owned())
             .collect::<Vec<String>>();
 
-        match dats.as_slice() {
-            [] => Ok(None),
-            [dat] => {
-                // only one .dat file, so use that
-                zip.by_name(dat)?.read_to_string(&mut data)?;
+        dats.into_iter()
+            .map(|name| {
+                let mut data = String::new();
+                zip.by_name(&name)?.read_to_string(&mut data)?;
                 let tree = Document::parse(&data)?;
-                Ok(Some(extra::dat_to_game(&tree)))
-            }
-            dats => {
-                // multiple .dat files, so use one with NS in filename
-                if let Some(dat) = dats.iter().find(|s| s.contains("NS")) {
-                    zip.by_name(dat)?.read_to_string(&mut data)?;
-                    let tree = Document::parse(&data)?;
-                    Ok(Some(extra::dat_to_game(&tree)))
-                } else {
-                    Ok(None)
-                }
-            }
-        }
+                Ok(extra::dat_to_game_db(&tree))
+            })
+            .collect()
     } else {
+        let mut data = String::new();
         f.read_to_string(&mut data)?;
         let tree = Document::parse(&data)?;
-        Ok(Some(extra::dat_to_game(&tree)))
+        Ok(vec![extra::dat_to_game_db(&tree)])
     }
 }
 
@@ -1545,7 +1700,7 @@ where
     create_dir_all(dir)?;
     let path = dir.join(db_file);
     let f = BufWriter::new(File::create(&path)?);
-    serde_cbor::to_writer(f, &cache).map_err(Error::CBOR)?;
+    serde_cbor::to_writer(f, &cache).map_err(Error::Cbor)?;
     println!("* Wrote \"{}\"", path.display());
     Ok(())
 }
@@ -1562,7 +1717,7 @@ where
         File::open(dirs.data_local_dir().join(db_file))
             .map_err(|_| Error::MissingCache(utility))?,
     );
-    serde_cbor::from_reader(f).map_err(Error::CBOR)
+    serde_cbor::from_reader(f).map_err(Error::Cbor)
 }
 
 fn verify(db: &game::GameDb, root: &Path, games: &HashSet<String>, only_failures: bool) {
