@@ -144,7 +144,7 @@ pub fn clear_tables(db: &Transaction) -> Result<(), Error> {
 
     db.execute("DELETE FROM SoftwareList", params![])
         .map(|_| ())
-        .map_err(Error::SQL)
+        .map_err(Error::Sql)
 }
 
 pub fn add_file(xml_path: &Path, db: &Transaction) -> Result<(), Error> {
@@ -155,7 +155,7 @@ pub fn add_file(xml_path: &Path, db: &Transaction) -> Result<(), Error> {
     File::open(xml_path).and_then(|mut f| f.read_to_string(&mut xml_file))?;
 
     let tree = Document::parse(&xml_file)?;
-    add_software_list(db, &tree.root_element()).map_err(Error::SQL)
+    add_software_list(db, &tree.root_element()).map_err(Error::Sql)
 }
 
 fn add_software_list(db: &Transaction, node: &Node) -> Result<(), rusqlite::Error> {
@@ -164,7 +164,7 @@ fn add_software_list(db: &Transaction, node: &Node) -> Result<(), rusqlite::Erro
         (name, description) VALUES (:name, :description)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":name": node.attribute("name"),
             ":description": node.attribute("description")
         })
@@ -188,7 +188,7 @@ fn add_software(
         (:softwarelist_id, :name, :cloneof, :supported)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":softwarelist_id": softwarelist_id,
             ":name": node.attribute("name"),
             ":cloneof": node.attribute("cloneof"),
@@ -219,7 +219,7 @@ fn add_description(db: &Transaction, software_id: i64, node: &Node) -> Result<()
          (software_id = :software_id)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":software_id": software_id,
             ":description": node.text()
         })
@@ -230,7 +230,7 @@ fn add_description(db: &Transaction, software_id: i64, node: &Node) -> Result<()
 fn add_year(db: &Transaction, software_id: i64, node: &Node) -> Result<(), rusqlite::Error> {
     db.prepare_cached("UPDATE Software SET year = :year WHERE (software_id = :software_id)")
         .and_then(|mut stmt| {
-            stmt.execute_named(named_params! {
+            stmt.execute(named_params! {
                 ":software_id": software_id,
                 ":year": node.text()
             })
@@ -244,7 +244,7 @@ fn add_publisher(db: &Transaction, software_id: i64, node: &Node) -> Result<(),
          (software_id = :software_id)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":software_id": software_id,
             ":publisher": node.text()
         })
@@ -258,7 +258,7 @@ fn add_info(db: &Transaction, software_id: i64, node: &Node) -> Result<(), rusql
         (:software_id, :name, :value)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":software_id": software_id,
             ":name": node.attribute("name"),
             ":value": node.attribute("value")
@@ -277,7 +277,7 @@ fn add_shared_feature(
         (:software_id, :name, :value)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":software_id": software_id,
             ":name": node.attribute("name"),
             ":value": node.attribute("value")
@@ -292,7 +292,7 @@ fn add_part(db: &Transaction, software_id: i64, node: &Node) -> Result<(), rusql
         (:software_id, :name, :interface)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":software_id": software_id,
             ":name": node.attribute("name"),
             ":interface": node.attribute("interface")
@@ -320,7 +320,7 @@ fn add_feature(db: &Transaction, part_id: i64, node: &Node) -> Result<(), rusqli
         (:part_id, :name, :value)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":part_id": part_id,
             ":name": node.attribute("name"),
             ":value": node.attribute("value")
@@ -335,7 +335,7 @@ fn add_data_area(db: &Transaction, part_id: i64, node: &Node) -> Result<(), rusq
         VALUES (:part_id, :name, :size, :width, :endianness)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":part_id": part_id,
             ":name": node.attribute("name"),
             ":size": node.attribute("size").map(|s| parse_int(s).map(|i| i as i64).expect("invalid data area size")),
@@ -358,7 +358,7 @@ fn add_rom(db: &Transaction, data_area_id: i64, node: &Node) -> Result<(), rusql
         VALUES
         (:data_area_id, :name, :size, :crc, :sha1,
          :offset, :value, :status, :loadflag)")
-    .and_then(|mut stmt| stmt.execute_named(
+    .and_then(|mut stmt| stmt.execute(
         named_params! {
             ":data_area_id": data_area_id,
             ":name": node.attribute("name"),
@@ -377,7 +377,7 @@ fn add_rom(db: &Transaction, data_area_id: i64, node: &Node) -> Result<(), rusql
 fn add_disk_area(db: &Transaction, part_id: i64, node: &Node) -> Result<(), rusqlite::Error> {
     db.prepare_cached("INSERT INTO DiskArea (part_id, name) VALUES (:part_id, :name)")
         .and_then(|mut stmt| {
-            stmt.execute_named(named_params! {
+            stmt.execute(named_params! {
                 ":part_id": part_id,
                 ":name": node.attribute("name")
             })
@@ -396,7 +396,7 @@ fn add_disk(db: &Transaction, disk_area_id: i64, node: &Node) -> Result<(), rusq
         VALUES (:disk_area_id, :name, :sha1, :status, :writeable)",
     )
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":disk_area_id": disk_area_id,
             ":name": node.attribute("name"),
             ":sha1": node.attribute("sha1"),
@@ -412,7 +412,7 @@ fn add_dipswitch(db: &Transaction, part_id: i64, node: &Node) -> Result<(), rusq
         "INSERT INTO Dipswitch (part_id, name, tag, mask) VALUES
         (:part_id, :name, :tag, :mask)")
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":part_id": part_id,
             ":name": node.attribute("name"),
             ":tag": node.attribute("tag"),
@@ -432,7 +432,7 @@ fn add_dipvalue(db: &Transaction, dipswitch_id: i64, node: &Node) -> Result<(),
         "INSERT INTO Dipvalue (dipswitch_id, name, value, switch_default) VALUES
         (:dipswitch_id, :name, :value, :default)")
     .and_then(|mut stmt| {
-        stmt.execute_named(named_params! {
+        stmt.execute(named_params! {
             ":dipswitch_id": dipswitch_id,
             ":name": node.attribute("name"),
             ":value": node.attribute("value").map(|s| parse_int(s).map(|i| i as i64).expect("invalid dipswitch value")),
@@ -476,33 +476,29 @@ pub fn xml_to_game_db(split_db: &mut SplitDb, tree: &Document) -> (String, GameD
 
 fn xml_to_game(node: &Node) -> Game {
     fn node_to_rom(node: &Node) -> Option<(String, Part)> {
-        if node.tag_name().name() == "rom" {
-            if let Some(sha1) = node.attribute("sha1") {
-                Some((
-                    node.attribute("name").unwrap().to_string(),
-                    Part::new_rom(sha1),
-                ))
-            } else {
-                None
-            }
-        } else {
-            None
-        }
+        (node.tag_name().name() == "rom")
+            .then(|| {
+                node.attribute("sha1").map(|sha1| {
+                    (
+                        node.attribute("name").unwrap().to_string(),
+                        Part::new_rom(sha1),
+                    )
+                })
+            })
+            .flatten()
     }
 
     fn node_to_disk(node: &Node) -> Option<(String, Part)> {
-        if node.tag_name().name() == "disk" {
-            if let Some(sha1) = node.attribute("sha1") {
-                Some((
-                    Part::name_to_chd(node.attribute("name").unwrap()),
-                    Part::new_disk(sha1),
-                ))
-            } else {
-                None
-            }
-        } else {
-            None
-        }
+        (node.tag_name().name() == "disk")
+            .then(|| {
+                node.attribute("sha1").map(|sha1| {
+                    (
+                        Part::name_to_chd(node.attribute("name").unwrap()),
+                        Part::new_disk(sha1),
+                    )
+                })
+            })
+            .flatten()
     }
 
     let mut game = Game {
@@ -664,8 +660,8 @@ pub fn parse_int(s: &str) -> Result<u64, ParseIntError> {
     u64::from_str(s)
         .or_else(|_| u64::from_str_radix(s, 16))
         .or_else(|e| {
-            if s.starts_with("0x") {
-                u64::from_str_radix(&s[2..], 16)
+            if let Some(stripped) = s.strip_prefix("0x") {
+                u64::from_str_radix(stripped, 16)
             } else {
                 dbg!(s);
                 Err(e)
